//#define DEBUG_MODE

using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Infragistics.Win;
using Infragistics.Win.UltraWinListView;

namespace Infragistics.Win.Samples
{
	#region DragDropManager class
	/// <summary>
	/// Class which implements drag/drop functionality for an Infragistics control.
	/// </summary>
	public class DragDropManager : IDisposable
	{
		#region Constants
		
		/// <summary>
		/// Represents a Point struct which has not been explicitly set.
		/// </summary>
		private static readonly Point			uninitializedPoint = new Point( -1, -1 );
		
		/// <summary>
		/// The numeric code for the SHIFT key (4).
		/// </summary>
		public static readonly int				numericCodeShift = 0x0004;

		/// <summary>
		/// The numeric code for the CONTROL key (8).
		/// </summary>
		public static readonly int				numericCodeControl = 0x0008;

		/// <summary>
		/// The numeric code for the ALT key (32).
		/// </summary>
		public static readonly int				numericCodeAlt = 0x0020;

		/// <summary>
		/// Defines the drag/drop effects which are supported by this class (Copy, Move)
		/// </summary>
		public static readonly DragDropEffects	supportedDragDropEffects = DragDropEffects.Copy | DragDropEffects.Move;

		#endregion Constants

		#region Member variables
		
		private Point							_lastMouseDown = DragDropManager.uninitializedPoint;
		private object							_dragData = null;
		private DragDropEffects					_dragDropEffect = DragDropEffects.None;
		private DragImageManager				_dragImageManager = null;
		private DragDropManager.IDragSource		_dragSource = null;
		private DragDropManager.IDropTarget		_dropTarget = null;

		#endregion Member variables

		#region Constructors

		/// <summary>
		/// Creates a new instance of the <see cref="DragDropManager"/> class.
		/// </summary>
		/// <param name="dragSource">The IDragSource implementation which handles drag functionality.</param>
		/// <param name="dropTarget">The IDropTarget implementation which handles drop functionality.</param>
		public DragDropManager( DragDropManager.IDragSource dragSource, DragDropManager.IDropTarget dropTarget )
		{
			this.Initialize( dragSource, dropTarget );
		}

		/// <summary>
		/// Creates a new instance of the <see cref="DragDropManager"/> class.
		/// </summary>
		/// <param name="dragSource">The IDragSource implementation which handles drag functionality.</param>
		public DragDropManager( DragDropManager.IDragSource dragSource )
		{
			this.Initialize( dragSource, null );
		}

		/// <summary>
		/// Creates a new instance of the <see cref="DragDropManager"/> class.
		/// </summary>
		/// <param name="dropTarget">The IDropTarget implementation which handles drop functionality.</param>
		public DragDropManager( DragDropManager.IDropTarget dropTarget )
		{
			this.Initialize( null, dropTarget );
		}

		private void Initialize( DragDropManager.IDragSource dragSource, DragDropManager.IDropTarget dropTarget )
		{
			if ( dragSource == null && dropTarget == null )
				throw new Exception( "Cannot create an instance of the DragDropManager class without either a drag source or a drop target." );

			//	Assign the member variables to the values of their respective constructor parameters
			this._dragSource = dragSource;
			this._dropTarget = dropTarget;

			//	If this instance serves as a drop target, set the AllowDrop
			//	property of the associated control to true.
			if ( this.SupportsDrop )
				this._dropTarget.Control.AllowDrop = true;

			//	Hook events.
			this.HookEvents( true );
		}

		#endregion Constructors

		#region Properties

			#region Control
		/// <summary>
		/// Returns the UltraControlBase-derived control associated with this instance.
		/// </summary>
		public UltraControlBase Control
		{
			get
			{ 
				if ( this._dragSource != null )
					return this._dragSource.Control;
				else
				if ( this._dropTarget!= null )
					return this._dropTarget.Control;
				else
					return null;
			}
		}
			#endregion Control
		
			#region SupportsDrag
		/// <summary>
		/// Returns whether the control associated with this instance can act as a drag source.
		/// </summary>
		public bool SupportsDrag
		{
			get{ return (this._dragSource != null ); }
		}
			#endregion SupportsDrag

			#region SupportsDrag
		/// <summary>
		/// Returns whether the control associated with this instance can act as a drop target.
		/// </summary>
		public bool SupportsDrop
		{
			get{ return (this._dropTarget != null ); }
		}
			#endregion SupportsDrop

		#endregion Properties

		#region Methods

			#region HookEvents
		/// <summary>
		/// Hooks events of interest.
		/// </summary>
		/// <param name="hook">Specifies whether to hook or unhook the events.</param>
		private void HookEvents( bool hook )
		{
			UltraControlBase dragControl = this._dragSource != null ? this._dragSource.Control : null;
			UltraControlBase dropControl = this._dropTarget != null ? this._dropTarget.Control : null;

			if ( hook )
			{
				if ( dragControl != null )
				{
					dragControl.MouseDown += new MouseEventHandler( this.OnMouseDown );
					dragControl.MouseMove += new MouseEventHandler( this.OnMouseMove );
					dragControl.QueryContinueDrag += new QueryContinueDragEventHandler( this.OnQueryContinueDrag );
					dragControl.GiveFeedback += new GiveFeedbackEventHandler( this.OnGiveFeedback );
				}

				if ( dropControl != null )
				{
					dropControl.DragOver += new DragEventHandler( this.OnDragOver );
					dropControl.DragDrop += new DragEventHandler( this.OnDragDrop );
				}
			}
			else
			{
				if ( dragControl != null )
				{
					dragControl.MouseDown -= new MouseEventHandler( this.OnMouseDown );
					dragControl.MouseMove -= new MouseEventHandler( this.OnMouseMove );
					dragControl.QueryContinueDrag -= new QueryContinueDragEventHandler( this.OnQueryContinueDrag );
					dragControl.GiveFeedback -= new GiveFeedbackEventHandler( this.OnGiveFeedback );
				}

				if ( dropControl != null )
				{
					dropControl.DragOver -= new DragEventHandler( this.OnDragOver );
					dropControl.DragDrop -= new DragEventHandler( this.OnDragDrop );
				}
			}
		}
			#endregion HookEvents

			#region OnDragOperationCompleted
		/// <summary>
		/// Handles the completion of a drag operation.
		/// </summary>
		/// <param name="canceled">Specifies whether the operation was canceled.</param>
		private void OnDragOperationCompleted( bool canceled )
		{
			this._dragData = null;
			this._lastMouseDown = DragDropManager.uninitializedPoint;
			Cursor.Current = Cursors.Default;

			if ( this._dragImageManager != null )
				this._dragImageManager.HideDragImage();
		}
			#endregion OnDragOperationCompleted

		#endregion Methods

		#region Event Handlers

			#region OnMouseDown
		/// <summary>
		/// Handles the MouseDown event of the associated control.
		/// This method records the location at which the left mouse button was pressed,
		/// so that the OnMouseMouse logic can determine whether a drag operation should begin.
		/// </summary>
		private void OnMouseDown( object sender, MouseEventArgs e )
		{
			if ( e.Button == MouseButtons.Left )
			{
				bool proceed = true;

				//	If the IDragSource interface implementor specified a UIElement type for
				//	hit testing, make sure the cursor is within the bounds of one before proceeding.
				Type hitTestElementType = this._dragSource != null ? this._dragSource.HitTestElementType : null;
				if ( hitTestElementType != null )
				{
					IUltraControlElement interfaceRef = this.Control as IUltraControlElement;
					ControlUIElementBase controlElement = interfaceRef != null ? interfaceRef.MainUIElement : null;
					Point clientPos = new Point( e.X, e.Y );
					UIElement elementAtPoint = controlElement != null ? controlElement.ElementFromPoint( clientPos ) : null;

					//	Walk up the element's ancestor chain in search of an element
					//	of the same (or derived) type as that specified by the creator
					//	of this instance.
					while ( elementAtPoint != null )
					{
						if ( elementAtPoint.GetType() == hitTestElementType ||
							 elementAtPoint.GetType().IsSubclassOf(hitTestElementType) )
							break;

						elementAtPoint = elementAtPoint.Parent;
					}

					proceed = elementAtPoint != null;
				}

				if ( proceed )
					this._lastMouseDown = new Point( e.X, e.Y );
			}
		}
			#endregion OnMouseDown

			#region OnMouseMove
		/// <summary>
		/// Handles the MouseMove event of the associated control.
		/// This method compares the location at which the left mouse button was pressed,
		/// to the size of the system-defined drag rectangle and initiates a drag operation
		/// if the point lies outside those bounds.
		/// </summary>
		private void OnMouseMove( object sender, MouseEventArgs e )
		{
			if ( this._dragSource == null )
				return;

			//	If the left mouse button is not pressed, return
			if ( (e.Button & MouseButtons.Left) == 0 )
				return;

			Point lastMouseDown = this._lastMouseDown;
			if ( lastMouseDown.X != DragDropManager.uninitializedPoint.X &&
				 lastMouseDown.Y != DragDropManager.uninitializedPoint.Y )
			{

				//	Determine the size and location of the drag rectangle; if
				//	the current mouse position is outside this rectangle, we will
				//	initiate a drag operation.
				Size dragSize = SystemInformation.DragSize;
				Rectangle dragRect = new Rectangle( lastMouseDown, dragSize );
				dragRect.X -= (dragSize.Width / 2);
				dragRect.Y -= (dragSize.Height / 2);

				//	If the current cursor position is within the bounds of the
				//	drag rectangle, fire the DragOperationStarting event.
				Point cursorPos = new Point( e.X, e.Y );
				if ( dragRect.Contains(cursorPos) == false )
				{
					//	Call the IDragSource interface's OnDragOperationStarting method
					DragOperationStartingInfo info = new DragOperationStartingInfo();
					this._dragSource.OnDragOperationStarting( info );

					//	If the event was not canceled, and drag data was assigned by
					//	a listener of the event, cache the drag data, which we will
					//	need later if the drop occurs.
					if ( info.CancelResolved == false )
					{
						//	Cache the drag data
						this._dragData = info.Data;

						//	If the DragImageElements of the event arguments was set,
						//	Create the drag image and use the DragDropIndicatorManager
						//	to display it.
						UltraListViewItem[] dragElements = info.DragImageElements;
						if ( dragElements != null )
						{
							if ( this._dragImageManager == null )
								this._dragImageManager = new DragImageManager();

							this._dragImageManager.Initialize( dragElements );
						}

						//	Call the DoDragDrop method to begin the drag operation
						this.Control.DoDragDrop( this._dragData, DragDropManager.supportedDragDropEffects );
					}
					else
						this.OnDragOperationCompleted( true );
				}
			}
		}
			#endregion OnMouseMove

			#region OnQueryContinueDrag
		/// <summary>
		/// Handles the QueryContinueDrag event of the associated control.
		/// </summary>
		private void OnQueryContinueDrag( object sender, QueryContinueDragEventArgs e )
		{
			if ( this._dragSource == null )
				return;

			//	If the DragOperationContinuing event was canceled, our reference
			//	to the the drag data will have been nullified, so terminate the
			//	drag operation if that is the case.
			if ( this._dragData == null )
				e.Action = DragAction.Cancel;

			//	If the Action is 'Cancel' or 'Drop', fire the DragOperationCompleting event
			if ( e.Action == DragAction.Cancel || e.Action == DragAction.Drop )
			{
				//	Call the IDragSource interface's OnDragOperationCompleting method
				DragOperationCompletingInfo info = new DragOperationCompletingInfo( this._dragData, this._dragDropEffect, e.Action == DragAction.Drop );
				this._dragSource.OnDragOperationCompleting( info );

				//	If the IDragSource interface implementor canceled the operation,
				//	set the Action property of the QueryContinueDragEventArgs to 'Cancel'.
				if ( info.Cancel )
					e.Action = DragAction.Cancel;

				this.OnDragOperationCompleted( e.Action == DragAction.Cancel );
			}
			//	If the Action is 'Continue' and we are displaying a drag image,
			//	move it to the new cursor position
			else
			{
				//	If we are displaying a drag image, move it to the new cursor position
				if ( this._dragImageManager != null )
					this._dragImageManager.ShowDragImage( System.Windows.Forms.Control.MousePosition );
			}
		}
			#endregion OnQueryContinueDrag

			#region OnGiveFeedback
		/// <summary>
		/// Handles the GiveFeedback event of the associated control.
		/// </summary>
		private void OnGiveFeedback( object sender, GiveFeedbackEventArgs e )
		{
			if ( this._dragSource == null )
				return;

			//	Call the IDragSource interface's OnDragOperationContinuing method
			DragOperationContinuingInfo info = new DragOperationContinuingInfo( this._dragData, e.Effect );
			this._dragSource.OnDragOperationContinuing( info );

			//	If the drag operation was canceled, nullify our reference to the
			//	drag data, so that the OnQueryContinueDrag logic knows not to continue.
			if ( info.Cancel )
				this._dragData = null;
			
			//	Set the cursors, if the IDragSource implementation returns specific ones.
			Cursor dragMoveCursor = this._dragSource.DragMoveCursor;
			Cursor dragCopyCursor = this._dragSource.DragCopyCursor;

			if ( info.Effect == DragDropEffects.Copy && dragCopyCursor != null )
			{
				e.UseDefaultCursors = false;
				Cursor.Current = dragCopyCursor;
			}
			else
			if ( info.Effect == DragDropEffects.Move && dragMoveCursor != null )
			{
				e.UseDefaultCursors = false;
				Cursor.Current = dragMoveCursor;
			}
			else
				e.UseDefaultCursors = true;

			//	Cache the last Effect setting so we know where we stand
			//	when we call the OnDragOperationCompleting method.
			this._dragDropEffect = e.Effect;

		}
			#endregion OnGiveFeedback

			#region OnDragOver
		/// <summary>
		/// Handles the DragOver event of the associated control.
		/// </summary>
		private void OnDragOver( object sender, DragEventArgs e )
		{
			if ( this._dropTarget == null )
				return;

			//	Call the IDropTarget interface implementor's OnDropTargetNotify method.
			DropTargetNotifyInfo info = new DropTargetNotifyInfo( e, DropTargetNotification.DragOver );
			this._dropTarget.OnDropTargetNotify( info );			
		}
			#endregion OnDragOver

			#region OnDragDrop
		/// <summary>
		/// Handles the DragDrop event of the associated control.
		/// </summary>
		private void OnDragDrop( object sender, DragEventArgs e )
		{
			if ( this._dropTarget == null )
				return;

			//	Call the IDropTarget interface implementor's OnDropTargetNotify method.
			DropTargetNotifyInfo info = new DropTargetNotifyInfo( e, DropTargetNotification.DragDrop );
			this._dropTarget.OnDropTargetNotify( info );			
		}
			#endregion OnDragDrop

		#endregion Event Handlers

		#region IDisposable
		/// <summary>
		/// Disposes of this instance.
		/// </summary>
		public void Dispose()
		{
			//	Unhook from the event notification list
			this.HookEvents( false );

			//	Dispose of the DragImageManager
			if ( this._dragImageManager != null )
			{
				this._dragImageManager.Dispose();
				this._dragImageManager = null;
			}
		}
		#endregion IDisposable

		#region DragImageManager class
		/// <summary>
		///	Generates the image displayed by a drag source (and its associated
		///	non-transparent region) while a drag operation is taking place.
		/// </summary>
		public class DragImageManager : IDisposable
		{
			#region Constants

			private static readonly Point			dragImageOffset = new Point( 2, 2 );
			private static readonly float			dragImageAlphaFactor = .33f;

			#endregion Constants

			#region Member variables

			private DragDropIndicatorManager		_dragDropIndicatorManager = null;
			private Bitmap							_dragImage = null;
			private Region							_dragImageRegion = null;

			#endregion Member variables

			#region Constructor
			/// <summary>
			/// Creates a new instance of the <see cref="DragImageManager"/> class.
			/// </summary>
			public DragImageManager()
			{
			}
			#endregion Constructor

			#region Methods

				#region Initialize
			/// <summary>
			/// Initializes this instance.
			/// </summary>
			/// <param name="dragElements">An array of UIElements from which an image is generated to depict the contents of the drag operation.</param>
			public void Initialize( UltraListViewItem[] dragElements )
			{
				//	Dispose of this instance, which will release any previously
				//	used resources.
				this.Dispose();

				//	Create the drag image and its region
				this.CreateDragImage( dragElements );
			}
				#endregion Initialize

				#region CreateDragImage
			/// <summary>
			/// Generates the image that is displayed to depict the contents of the drag operation.
			/// </summary>
			/// <param name="dragElements">An array of UIElements from which the image is generated.</param>
            private void CreateDragImage(UltraListViewItem[] dragElements)
			{
				if ( dragElements == null || dragElements.Length == 0 )
					return;

                Size size = new Size(dragElements.Length * 10 + 50, dragElements.Length * 10 + 50);//new Size( right - left, bottom - top );
				Rectangle bounds = new Rectangle( Point.Empty, size );

				//	Create the Bitmap which we will draw on to create the drag image
                this._dragImage = new Bitmap( size.Width, size.Height );
				Bitmap img = new Bitmap( size.Width, size.Height );
               
				Graphics gr = null;
				try
				{
					//	Create a Graphics object from the Bitmap we created above;
					//	fill it with the transparent color and offset its rendering origin.
					gr = Graphics.FromImage( this._dragImage );

					//	Get the BackColor of the parent form...we need to use this as the background color
					//	because when we draw the UIElements to the Graphics object, transparent areas will
					//	be drawn with this color, and we want those areas to be masked out of the resulting image.
					Color parentFormBackColor = dragElements[0] != null ? dragElements[0].Control.BackColor : Form.DefaultBackColor;
					gr.Clear( parentFormBackColor );
                    
					//	Draw each element onto the Graphics object
                    Point location = new Point(1,1);
					for ( int i = 0; i < dragElements.Length; i ++ )
					{
                        UltraListViewItem Item = dragElements[i] as UltraListViewItem;
                        gr.DrawImage(GetImageFromItem(Item), location);
                        location.Offset(10,10);
					}

					//	Generate a Region from the Bitmap we just created above
					this._dragImageRegion = ImageManager.CreateBitmapRegion( this._dragImage, parentFormBackColor );

					//	Adjust the opacity of the drag image to give it
					//	a semi-transparent, "ghosted" appearance.
					this.AdjustTransparency( this._dragImage );
				}
				finally
				{
					//	Dispose of the Graphics and Matrix objects
					if ( gr != null )
						gr.Dispose();
                    
				}
			}


            Bitmap GetImageFromItem(UltraListViewItem Item)
            {
                Bitmap img = Item.Appearance.Image as Bitmap;
                
                int width = img.Width + 14;
                int height = img.Height + 15;
                Bitmap retImg = new Bitmap(width, height);
                string text = Item.Text;
                Font f ;
                if(Item.Appearance.HasFontData)
                    f = new Font(Item.Appearance.FontData.Name,Item.Appearance.FontData.SizeInPoints);
                else
                    f = new Font("Tahuma",7, FontStyle.Regular);
                Brush b = new SolidBrush(Color .Black);
                Graphics g = Graphics.FromImage(retImg);
               
                Point location = new Point(7, 0);//new Point(Math.Abs(Item.Rect.Location.X - img.Rect.Location.X), Math.Abs(Item.Rect.Location.Y - img.Rect.Location.Y));
                g.DrawImage(img, location);
                g.DrawString(text, f, b, new PointF(0, img.Height));
                f.Dispose();
                b.Dispose();
                g.Dispose();
                return retImg;
            }

				#endregion CreateDragImage
			
				#region AdjustTransparency
			/// <summary>
			/// Adjusts the transparency of the specified image.
			/// </summary>
			/// <param name="dragImage">The image to adjust</param>
			private void AdjustTransparency( Bitmap dragImage )
			{
				if ( dragImage == null )
					return;

				BitmapData bitmapData = null;
				try
				{
					Rectangle rect = new Rectangle( Point.Empty, dragImage.Size );
					int width = dragImage.Size.Width;
					int height = dragImage.Size.Height;
					bitmapData = dragImage.LockBits( rect, ImageLockMode.ReadWrite, dragImage.PixelFormat );
					byte[] pixelData = new byte[ rect.Height * bitmapData.Stride ];
					int pixelSize = bitmapData.Stride / width;

					// Get the pixel data into the byte array
					Marshal.Copy( bitmapData.Scan0, pixelData, 0, pixelData.Length );

					//	Iterate the scan lines of the bitmap, and reduce the opacity
					//	of each pixel to make them semi-transparent.
					for ( int y = 0; y < height; y ++ )
					{
						int offset = y * bitmapData.Stride;
						
						for ( int x = 0; x < width; x ++ )
						{
							byte alpha = pixelData[offset + 3];
							if ( alpha == 0 )
								continue;

							pixelData[offset + 3] = (byte)(alpha * DragImageManager.dragImageAlphaFactor);
							
							offset += pixelSize;
						}
					}

					// Copy the manipulated bytes back into the bitmap
					Marshal.Copy( pixelData, 0, bitmapData.Scan0, pixelData.Length );

				}
				finally
				{
					if ( bitmapData != null )
						dragImage.UnlockBits( bitmapData );
				}
			}
				#endregion AdjustTransparency

				#region ShowDragImage
			/// <summary>
			/// Shows the drag image at the specified location.
			/// </summary>
			/// <param name="location">The location at which to display the image, expressed in coordinates relative to the screen.</param>
			public void ShowDragImage( Point location )
			{
				if ( this._dragImage == null )
					return;

				if ( this._dragDropIndicatorManager == null )
				{
					//	Note that the constructor parameters are meaningless here (there is no parameterless ctor).
					this._dragDropIndicatorManager = new DragDropIndicatorManager( Orientation.Horizontal, null );
					this._dragDropIndicatorManager.InitializeDragIndicator( this._dragImage, this._dragImageRegion );
					this._dragDropIndicatorManager.DragIndicatorOffset = DragImageManager.dragImageOffset;
				}

				this._dragDropIndicatorManager.ShowDragIndicator( location );
			}
				#endregion ShowDragImage

				#region HideDragImage
			/// <summary>
			/// Hides the drag image.
			/// </summary>
			public void HideDragImage()
			{
				if ( this._dragDropIndicatorManager != null )
					this._dragDropIndicatorManager.HideDragIndicator();

				this.Dispose();
			}
				#endregion HideDragImage

			#endregion Methods

			#region IDisposable
			/// <summary>
			/// Disposes of this instance.
			/// </summary>
			public void Dispose()
			{
				//	Dispose of the DragDropIndicatorManager
				if ( this._dragDropIndicatorManager != null )
				{
					if ( this._dragDropIndicatorManager.Disposed == false )
					{
						if ( this._dragDropIndicatorManager.IsDragIndicatorVisible )
							this._dragDropIndicatorManager.HideDragIndicator();

						this._dragDropIndicatorManager.Dispose();
					}

					this._dragDropIndicatorManager = null;
				}

				//	Dispose of the drag image
				if ( this._dragImage != null )
				{
					this._dragImage.Dispose();
					this._dragImage = null;
				}

				//	Dispose of the region we created from the drag image
				if ( this._dragImageRegion != null )
				{
					this._dragImageRegion.Dispose();
					this._dragImageRegion = null;
				}
			}
			#endregion IDisposable

		}
		#endregion DragImageManager class

		#region Drag/drop information related classes

			#region DragOperationInfoBase
		/// <summary>
		/// Base class for the drag operation classes.
		/// </summary>
		public class DragOperationInfoBase
		{
			private bool _cancel = false;

			/// <summary>
			/// Gets/sets whether the drag operation should be canceled.
			/// </summary>
			public bool Cancel
			{
				get{ return this._cancel; }
				set{ this._cancel = value; }
			}

		}
			#endregion DragOperationInfoBase

			#region DragOperationStartingInfo class
		/// <summary>
		/// Class which contains information related to the beginning of a drag/drop operation.
		/// </summary>
		public class DragOperationStartingInfo : DragOperationInfoBase
		{
			private object					_data = null;
            private UltraListViewItem[] _dragImageElements = null;

			/// <summary>
			/// Gets/sets the data associated with this drag operation.
			/// </summary>
			public object Data
			{ 
				get{ return this._data; } 
				set{ this._data = value; } 
			}

			/// <summary>
			/// Gets/sets the array of UIElements which will be used to render the
			/// image that represents the contents of the drag operation.
			/// </summary>
            public UltraListViewItem[] DragImageElements
			{
				get{ return this._dragImageElements; } 
				set{ this._dragImageElements = value; } 
			}

			/// <summary>
			/// Returns whether the event has been logically canceled,
			/// i.e., whether the 'Cancel' property has been set to true,
			/// there is no data to drop, or no drag operation type was specified.
			/// </summary>
			public bool CancelResolved
			{
				get{ return this.Cancel == true || this.Data == null; }
			}
		}
			#endregion DragOperationStartingInfo class
		
			#region DragOperationContinuingInfo class
		/// <summary>
		/// Class which contains information related to a continuing drag/drop operation.
		/// </summary>
		public class DragOperationContinuingInfo : DragOperationInfoBase
		{
			private object					_data = null;
			private DragDropEffects			_effect = DragDropEffects.None;

			/// <summary>
			/// Constructor
			/// </summary>
			public DragOperationContinuingInfo( object dragData, DragDropEffects effect )
			{
				this._data = dragData;
				this._effect = effect;
			}

			/// <summary>
			/// Gets/sets the data associated with this drag operation.
			/// </summary>
			public object Data
			{ 
				get{ return this._data; } 
			}

			/// <summary>
			/// Returns the DragDropEffects
			/// </summary>
			public DragDropEffects Effect
			{
				get{ return this._effect; }
			}
		}
			#endregion DragOperationContinuingInfo class
		
			#region DragOperationCompletingInfo class
		/// <summary>
		/// Class which contains information related to the completion of a drag/drop operation.
		/// </summary>
		public class DragOperationCompletingInfo : DragOperationInfoBase
		{
			private object					_data = null;
			private DragDropEffects			_effect = DragDropEffects.None;
			private bool					_committed = true;

			/// <summary>
			/// Constructor
			/// </summary>
			public DragOperationCompletingInfo( object dragData, DragDropEffects effect, bool committed )
			{
				this._data = dragData;
				this._effect = effect;
				this._committed = committed;
			}

			/// <summary>
			/// Gets/sets the data associated with this drag operation.
			/// </summary>
			public object Data
			{ 
				get{ return this._data; } 
			}

			/// <summary>
			/// Returns the DragDropEffects
			/// </summary>
			public DragDropEffects Effect
			{
				get{ return this._effect; }
			}

			/// <summary>
			/// Returns a boolean indicating whether the drag operation was committed,
			/// i.e., none of the other cancelable events were canceled, and the end user
			/// did not press the Escape key.
			/// </summary>
			public bool Committed
			{
				get{ return this._committed; }
			}
		}
			#endregion DragOperationCompletingInfo class
		
			#region DropTargetNotifyInfo class
		/// <summary>
		/// Event arguments class for the DragDropManager's 'DropTargetNotify' event.
		/// </summary>
		public class DropTargetNotifyInfo
		{
			private DragEventArgs			_dragEventArgs = null;
			private DropTargetNotification	_dropTargetNotification = DropTargetNotification.None;

			/// <summary>
			/// Constructor
			/// </summary>
			public DropTargetNotifyInfo( DragEventArgs dragEventArgs, DropTargetNotification dropTargetNotification )
			{
				if ( dragEventArgs == null )
					throw new ArgumentNullException( "dragEventArgs" );

				this._dragEventArgs = dragEventArgs;
				this._dropTargetNotification = dropTargetNotification;
			}

			/// <summary>
			/// Gets/sets the DragDropEffects constant which describes the
			/// drag/drop effects which should be allowed for this operation.
			/// </summary>
			public DragDropEffects Effect
			{
				get{ return this._dragEventArgs.Effect; }
				set{ this._dragEventArgs.Effect = value; }
			}

			/// <summary>
			/// Gets/sets the IDataObject implementor which contains
			/// the data associated with this drag operation.
			/// </summary>
			public IDataObject Data
			{ 
				get{ return this._dragEventArgs.Data; } 
			}

			/// <summary>
			/// Returns which drag/drop operations are allowed by the drag source.
			/// </summary>
			public DragDropEffects AllowedEffect
			{
				get{ return this._dragEventArgs.AllowedEffect; }
			}

			/// <summary>
			///	Returns whether the SHIFT key is currently pressed.
			/// </summary>
			public bool IsShiftKeyPressed
			{
				get{ return (this._dragEventArgs.KeyState & DragDropManager.numericCodeShift) == DragDropManager.numericCodeShift; }
			}

			/// <summary>
			///	Returns whether the ALT key is currently pressed.
			/// </summary>
			public bool IsAltKeyPressed
			{
				get{ return (this._dragEventArgs.KeyState & DragDropManager.numericCodeAlt) == DragDropManager.numericCodeAlt; }
			}

			/// <summary>
			///	Returns whether the CONTROL key is currently pressed.
			/// </summary>
			public bool IsControlKeyPressed
			{
				get{ return (this._dragEventArgs.KeyState & DragDropManager.numericCodeControl) == DragDropManager.numericCodeControl; }
			}

			/// <summary>
			/// Returns whether the notification was triggered by the
			/// DragOver event or the DragDrop event.
			/// </summary>
			public DropTargetNotification DropTargetNotification
			{
				get{ return this._dropTargetNotification; }
			}
		}
			#endregion DropTargetNotifyInfo class
		
		#endregion Drag/drop information related classes

		#region DropTargetNotification
		/// <summary>
		/// Constants which identify the type of drop target notification.
		/// </summary>
		public enum DropTargetNotification
		{
			/// <summary>
			/// No notification.
			/// </summary>
			None,

			/// <summary>
			/// The DragOver event.
			/// </summary>
			DragOver,

			/// <summary>
			/// The DragDrop event.
			/// </summary>
			DragDrop,
		}
		#endregion DragDropSupportedOperations enumeration

		#region IDragSource interface definition
		public interface IDragSource
		{
			/// <summary>
			/// Returns the UltraControlBase-derived control which acts as the drag source.
			/// </summary>
			UltraControlBase Control{ get; }

			/// <summary>
			/// Called when a drag operation is about to begin.
			/// </summary>
			/// <param name="info">The <see cref="DragOperationStartingInfo"/> instance which contains information relevant to the drag operation.</param>
			void OnDragOperationStarting( DragOperationStartingInfo info );

			/// <summary>
			/// Called when a drag operation is in progress.
			/// </summary>
			/// <param name="info">The <see cref="DragOperationContinuingInfo"/> instance which contains information relevant to the drag operation.</param>
			void OnDragOperationContinuing( DragOperationContinuingInfo info );

			/// <summary>
			/// Called when a drag operation is about to be completed.
			/// </summary>
			/// <param name="info">The <see cref="DragOperationCompletingInfo"/> instance which contains information relevant to the drag operation.</param>
			void OnDragOperationCompleting( DragOperationCompletingInfo info );

			/// <summary>
			/// Returns the type of the UIElement which is hit-tested before a drag
			/// operation is initiated; if this property returns a non-null value,
			/// a UIElement of this type must be found at the cursor position before
			/// a drag operation is initiated. Can return null, in which case the
			/// drag operation is initiated regardless of whether a draggable item
			/// is found at the cursor position.
			/// </summary>
			Type HitTestElementType{ get; }

			/// <summary>
			/// Returns the cursor that should be displayed during a drag move operation.
			/// Can return null, in which case the default cursor is displayed.
			/// </summary>
			Cursor DragMoveCursor{ get; }

			/// <summary>
			/// Returns the cursor that should be displayed during a drag copy operation.
			/// Can return null, in which case the default cursor is displayed.
			/// </summary>
			Cursor DragCopyCursor{ get; }

		}
		#endregion IDragSource interface definition

		#region IDropTarget interface definition
		public interface IDropTarget
		{
			/// <summary>
			/// Returns the UltraControlBase-derived control which acts as the drop target.
			/// </summary>
			UltraControlBase Control{ get; }

			/// <summary>
			/// Called to notify a drop target about the progress of a drag/drop operation
			/// </summary>
			/// <param name="info">The <see cref="DropTargetNotifyInfo"/> instance which contains information relevant to the drop.</param>
			void OnDropTargetNotify( DropTargetNotifyInfo info );
		}
		#endregion IDropTarget interface definition

	}
	#endregion DragDropManager class

}