using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Infragistics.Win.UltraWinListView;

namespace Infragistics.Win.Samples
{
	#region WinListViewDragSource
	/// <summary>
	/// Class which implements the IDragSource interface for the WinListView control.
	/// </summary>
	public class WinListViewDragSource : DragDropManager.IDragSource
	{
		#region Member variables
		
		private UltraListView _listView = null;
		private Cursor _dragMoveCursor = null;
		private Cursor _dragCopyCursor = null;
		
		#endregion Member variables
	
		#region Constructors

		/// <summary>
		/// Creates a new instance of the <see cref="WinListViewDragSource"/> class.
		/// </summary>
		/// <param name="listView">The UltraListView control with which this instance is associated.</param>
		public WinListViewDragSource( UltraListView listView ) : this( listView, null, null ){}

		/// <summary>
		/// Creates a new instance of the <see cref="WinListViewDragSource"/> class.
		/// </summary>
		/// <param name="listView">The UltraListView control with which this instance is associated.</param>
		/// <param name="dragMoveCursor">The cursor to display during a drag move operation.</param>
		/// <param name="dragCopyCursor">The cursor to display during a drag copy operation.</param>
		public WinListViewDragSource( UltraListView listView, Cursor dragMoveCursor, Cursor dragCopyCursor )
		{
			if ( listView == null )
				throw new ArgumentNullException( "listView" );

			this._listView = listView;
			this._dragMoveCursor = dragMoveCursor;
			this._dragCopyCursor = dragCopyCursor;
		}

		#endregion Constructors

		#region IDragSource interface implementation

		//	Implements the Control property.
		UltraControlBase DragDropManager.IDragSource.Control
		{
			get{ return this._listView; }
		}

		//	Implements the DragMoveCursor property.
		Cursor DragDropManager.IDragSource.DragMoveCursor
		{
			get{ return this._dragMoveCursor; }
		}

		//	Implements the DragCopyCursor property.
		Cursor DragDropManager.IDragSource.DragCopyCursor
		{
			get{ return this._dragCopyCursor; }
		}

		//	Implements the HitTestElementType property.
		Type DragDropManager.IDragSource.HitTestElementType
		{
			get{ return typeof(UltraListViewItemUIElement); }
		}

		//	Implements the OnDragOperationStarting method.
		void DragDropManager.IDragSource.OnDragOperationStarting( DragDropManager.DragOperationStartingInfo info )
		{
			UltraListView listView = this._listView;

			//	Get a snapshot of the SelectedItems collection into an array
			UltraListViewSelectedItemsCollection selectedItems = listView.SelectedItems;
			UltraListViewItem[] dragData = new UltraListViewItem[ selectedItems.Count ];
			selectedItems.CopyTo( dragData, 0 );

			//	Assign the UltraListViewItem array to the Drag property of the event arguments;
			//	this array will be passed around for the lifetime of the drag operation
			info.Data = dragData;
			
			//	Get another array, this one containing the UIElement for each of the
			//	selected items; these elements are used to generate an image which is
			//	displayed during the drag operation, so the end user can see what is
			//	being dragged. Assign this array to the 'DragImageElements' property
			//	of the event arguments.
			info.DragImageElements = WinListViewDragSource.GetDragImageElements( listView.View, dragData );			
		}

		//	Returns an array of UIElements which should be displayed in the drag image
		//	based on the specified view and UltraListViewItem array.
        static private UltraListViewItem[] GetDragImageElements(UltraListViewStyle view, UltraListViewItem[] items)
		{
            //if ( items == null || items.Length == 0 )
            //    return null;

            //ArrayList elementList = new ArrayList( view == UltraListViewStyle.Details ? items.Length * 2 : items.Length );

            //for ( int i = 0; i < items.Length; i ++ )
            //{
            //    UltraListViewItemUIElement itemElement = items[i].UIElement;
            //    if ( itemElement == null )
            //        continue;

            //    switch ( view )
            //    {
            //        case UltraListViewStyle.Details:
            //        {
            //            elementList.Add( itemElement.GetDescendant( typeof(UltraListViewImageUIElement) ) );
            //            elementList.Add( itemElement.GetDescendant( typeof(UltraListViewMainItemTextAreaUIElement) ) );
            //        }
            //        break;

            //        default:
            //        {
            //            elementList.Add( itemElement );
            //        }
            //        break;
            //    }
            //}

            return items;//elementList.ToArray( typeof(UIElement) ) as UIElement[];
		}

		//	Implements the OnDragOperationContinuing method.
		void DragDropManager.IDragSource.OnDragOperationContinuing( DragDropManager.DragOperationContinuingInfo info )
		{
			//	Note that this method can be handled to cancel the drag operation,
			//	or display non-default cursors. The DragDropManager's default
			//	implementation handles the display of the move and copy cursors,
			//	which is all we need for this demonstration, so we don't need to
			//	do anything else here.
		}

		//	Handles the DragOperationCompleting event
		void DragDropManager.IDragSource.OnDragOperationCompleting( DragDropManager.DragOperationCompletingInfo info )
		{
			UltraListView listView = this._listView;

			//	If the drag operation is being committed, and it is a move
			//	operation, remove the items from the drag source's Items collection.
			if ( info.Committed && info.Effect == DragDropEffects.Move )
			{
				UltraListViewItem[] items = info.Data as UltraListViewItem[];
				if ( items != null )
				{
					for ( int i = items.Length - 1; i >= 0; i -- )
					{
						listView.Items.Remove( items[i] );
					}
				}
			}
		}

		#endregion IDragSource interface implementation
	}
	#endregion WinListViewDragSource

}